package org.eclipse.swt.widgets;

import org.eclipse.swt.internal.cocoa.NSFileManager;
import org.eclipse.swt.internal.cocoa.NSString;
import org.eclipse.swt.internal.cocoa.OS;

/**
 * The purpose of this class is to fix https://support.centeractive.net/issues/3104 by performing the following change (mentioned in
 * https://support.centeractive.net/issues/3104)
 * http://git.eclipse.org/c/platform/eclipse.platform.swt.git/commit/?id=346e516fd49e5d732af4c82575d10fcf693d38c9 without recompiling SWT source code.
 *
 * Since this class references Mac specific classes it can be compiled only on Mac and cannot be committed as source to SVN. Therefore, the name of
 * this file is changed from FileDialogImprovedForMac.java to FileDialogImprovedForMac.jav. Thanks to that this file is excluded from java
 * compilation.
 *
 * So if you want to change this class do the following:
 * <ol>
 * <li>Rename this file from FileDialogImprovedForMac.jav to FileDialogImprovedForMac.java
 * <li>Perform the needed changes
 * <li>Manually copy com.retrospective.ui/target/classes/org/eclipse/swt/widgets/FileDialogImprovedForMac.class to
 * com.retrospective.ui/org/eclipse/swt/widgets/FileDialogImprovedForMac.class
 * <li>Rename back this file from FileDialogImprovedForMac.java to FileDialogImprovedForMac.jav
 * <li>Commit the changes
 * </ol>
 *
 * If you want to change the name or location of this file, then please update also remove-FileDialogImprovedForMac-before-obfuscation and
 * add-FileDialogImprovedForMac-after-obfuscation Ant macros in com.retrospective.product/pom.xml. These macros are removing
 * FileDialogImprovedForMac.class from ui jar file for the duration of obfuscation process (otherwise there were some strange errors).
 */
public class FileDialogImprovedForMac extends FileDialog {

    public FileDialogImprovedForMac(Shell parent, int style) {
        super(parent, style);
    }

    /**
     * The code of this method was taken from org.eclipse.swt.cocoa.macosx.x86_64_3.8.1.v3836b.jar:org.eclipse.swt.widgets/FileDialog class and then a
     * single line, marked below, was changed.
     */
    @Override
    long /* int */panel_shouldShowFilename(long /* int */id, long /* int */sel, long /* int */arg0, long /* int */arg1) {
        NSString path = new NSString(arg1);
        if (filterExtensions != null && filterExtensions.length != 0) {
            NSFileManager manager = NSFileManager.defaultManager();
            long /* int */ptr = OS.malloc(1);
            boolean found = manager.fileExistsAtPath(path, ptr);
            byte[] isDirectory = new byte[1];
            OS.memmove(isDirectory, ptr, 1);
            OS.free(ptr);
            if (found) {
                if (isDirectory[0] != 0) {
                    return 1;
                } else if (popup != null) { // This is the only line changed. Previously it was: "} else {" see
                                            // http://git.eclipse.org/c/platform/eclipse.platform.swt.git/commit/?id=346e516fd49e5d732af4c82575d10fcf693d38c9
                    String fileName = path.lastPathComponent().getString();
                    int filterIndex = (int) /* 64 */popup.indexOfSelectedItem();
                    String extensions = filterExtensions[filterIndex];
                    int start = 0, length = extensions.length();
                    while (start < length) {
                        int index = extensions.indexOf(EXTENSION_SEPARATOR, start);
                        if (index == -1)
                            index = length;
                        String filter = extensions.substring(start, index).trim();
                        if (filter.equals("*") || filter.equals("*.*"))
                            return 1;
                        if (filter.startsWith("*."))
                            filter = filter.substring(2);
                        if ((fileName.toLowerCase()).endsWith("." + filter.toLowerCase()))
                            return 1;
                        start = index + 1;
                    }
                    return 0;
                }
            }
        }
        return 1;
    }

}
